unit uRechFile;

interface

uses
  Forms, Classes {$IFDEF MSWINDOWS} , Windows {$ENDIF}, SysUtils, FileTools, SyncObjs;

type
  TNombreFichier = class(TThread)
  private
    FDossier: string;
    FNombreFichier: Integer;
    Lock: TCriticalSection;
    procedure SetName;
    procedure SetNombreFichier(const Value: Integer);
    function GetNombreFichier: Integer;

    function RechercheFile(const Dossier: string): boolean;
  protected
    procedure Execute; override;

  public
    constructor Create(Dossier: string);
    destructor Destroy(); override;
    property NombreFichier: Integer read GetNombreFichier write SetNombreFichier;
  end;

implementation

{ Important : les mthodes et proprits des objets de la VCL peuvent uniquement tre
  utiliss dans une mthode appele en utilisant Synchronize, comme : 

      Synchronize(UpdateCaption);

  o UpdateCaption serait de la forme

    procedure TNombreFichier.UpdateCaption;
    begin
      Form1.Caption := 'Mis  jour dans un thread';
    end; }

{$IFDEF MSWINDOWS}
type
  TThreadNameInfo = record
    FType: LongWord;     // doit tre 0x1000
    FName: PChar;        // pointeur sur le nom (dans l'espace d'adresse de l'utilisateur)
    FThreadID: LongWord; // ID de thread (-1=thread de l'appelant)
    FFlags: LongWord;    // rserv pour une future utilisation, doit tre zro
  end;
{$ENDIF}

{ TNombreFichier }

procedure TNombreFichier.SetName;
{$IFDEF MSWINDOWS}
var
  ThreadNameInfo: TThreadNameInfo;
{$ENDIF}
begin
{$IFDEF MSWINDOWS}
  ThreadNameInfo.FType := $1000;
  ThreadNameInfo.FName := 'Nombre de Fichier';
  ThreadNameInfo.FThreadID := $FFFFFFFF;
  ThreadNameInfo.FFlags := 0;

  try
    RaiseException( $406D1388, 0, sizeof(ThreadNameInfo) div sizeof(LongWord), @ThreadNameInfo );
  except
  end;
{$ENDIF}
end;

procedure TNombreFichier.Execute;
begin
  SetName;
  NombreFichier := 0;
  RechercheFile(FDossier);
end;

procedure TNombreFichier.SetNombreFichier(const Value: Integer);
begin
  Lock.Acquire;
  FNombreFichier := Value;
  Lock.Release;
end;

function TNombreFichier.GetNombreFichier: Integer;
begin
  Lock.Acquire;
  result := FNombreFichier;
  Lock.Release;
end;

function TNombreFichier.RechercheFile(const Dossier: string): boolean;
var res : integer;
    SRec: TSearchRec;
begin
  result := true;

  res := FindFirst(NormalDir(Dossier) + '*.*', faAnyFile, SRec);
  while(res = 0) do begin
    if(faDirectory and SRec.Attr > 0) then begin
      if((SRec.Name <> '.') and (SRec.Name <> '..')) then
        RechercheFile(NormalDir(Dossier) + SRec.Name);
    end else
      NombreFichier := NombreFichier + 1;
    res := FindNext(SRec);
  end;
  FindClose(SRec);
end;

constructor TNombreFichier.Create(Dossier: string);
begin
  inherited Create(true);
  FDossier := Dossier;
  Priority := tpTimeCritical;
  Lock := TCriticalSection.Create;
  Resume();
end;

destructor TNombreFichier.Destroy;
begin
  Lock.Free;
  inherited;
end;

end.
